%Simulates a 3-dimensional dissipative Ising model.
%At each site there are two spins X1 and X_2 which can be -1 or +1.
%The local magnetization M(site) is the average of Xi(nn) where i=1,2
%  and nn runs over all neighbors of the site.
%The spin X1 flips with rate exp(-beta*M(site)).
%The spin X2 flips with rate alpha times the indicator that X1=X2.

alpha=0.2;                 %model parameter
beta=8;                    %model parameter
L=8; 		           %size of the box
Times=linspace(0,60,61);   %times when the state is recorded

X=ones(L,L,L,2);           %constant initial state

begin=clock;

Steps=length(Times);
magnet=zeros(1,Steps);     %record total magnetization
M=zeros(L,L,L);            %local magnetization
R=zeros(L,L,L);            %local flip rate
for k=1:L
 for l=1:L
  for m=1:L
   nn=neigh([k l m],L);    %coordinates of the 6 neighboring sites
   for n=1:6               %add contributions of neighors to magnetization
    nu=nn(n,:);
    M(k,l,m)=M(k,l,m)+X(nu(1),nu(2),nu(3),1)+X(nu(1),nu(2),nu(3),2);
   end
   R(k,l,m)=exp(-beta*X(k,l,m,1)*M(k,l,m)/12); %local flip rate
  end
 end
end
AA=alpha*L*L*L;            %total dissipation rate
RR=sum(sum(sum(R)));       %total flip rate
tt=0;                      %duration of current step of the simulation.
time=0;                    %real time
step=1;
while step<=Steps          %while there is still a time frame to be recorded
 time=time+tt;             %increase real time
 if time>=Times(step)
  Film{1,step}=X;          %record the state
  magnet(step)=sum(sum(sum(sum(X))))/(L*L*L*2);  %record total magentization
  step=step+1;             %increase counter of next state to be recorded
 end
 if rand<AA/(AA+RR);       %dissipate
  ss=ceil(L*rand(1,3));    %choose random site
  if X(ss(1),ss(2),ss(3),2)==X(ss(1),ss(2),ss(3),1);
   X(ss(1),ss(2),ss(3),2)=-X(ss(1),ss(2),ss(3),2);
   nn=neigh(ss,L);         %coordinates of the 6 neighboring sites
   for n=1:6               %change the magnetization and flip rates
    nu=nn(n,:);            %of neighbors
    M(nu(1),nu(2),nu(3))=M(nu(1),nu(2),nu(3))+2*X(ss(1),ss(2),ss(3),2);
    R(nu(1),nu(2),nu(3))=exp(-beta*X(nu(1),nu(2),nu(3),1)*M(nu(1),nu(2),nu(3))/12);
   end
  end
 else                      %flip
  ss=choose3(R/RR);        %choose proportional to flip rate
  X(ss(1),ss(2),ss(3),1)=-X(ss(1),ss(2),ss(3),1);
  R(ss(1),ss(2),ss(3))=1/R(ss(1),ss(2),ss(3)); %change flip rate
  nn=neigh(ss,L);          %coordinates of the 6 neighboring sites
  for n=1:6                %change the magnetization and flip rates
   nu=nn(n,:);             %of neighbors
   M(nu(1),nu(2),nu(3))=M(nu(1),nu(2),nu(3))+2*X(ss(1),ss(2),ss(3),1);
   R(nu(1),nu(2),nu(3))=exp(-beta*X(nu(1),nu(2),nu(3),1)*M(nu(1),nu(2),nu(3))/12);
  end
 end
 RR=sum(sum(sum(R)));      %update total flip rate
 tt=1/(AA+RR);             %size of the time step is inverse of total rate
end

elaps=etime(clock,begin);
elaps=elaps/(60*60);
hours=floor(elaps)
elaps=60*(elaps-hours);
minutes=floor(elaps)
seconds=60*(elaps-minutes)

for step=1:Steps    %a bit of post processing to get the right colors
 X=Film{1,step};    %local state (X1,X2) is translated into (Y1,Y2)
 Z=X;               %where Y1=1,2 if X1=-1,+1, and
 Z(:,:,:,2)=X(:,:,:,1).*X(:,:,:,2); %Y2=2 when X1=X2 and Y2=1 when X1!=X2.
 Film{1,step}=1+(Z+1)/2;  %so Y2=2 for good, Y2=1 for corrupted
end

step=1;
q=[2 2];                     %number of types
Pic=makepic(Film{1,step},q); %Show the initial state
imshow(Pic)

%To plot the total magnetization as a function of time, type:
%plot(Times,magnet)